<?php

namespace App\Http\Controllers;

use App\Models\Card;
use App\Models\Course;
use App\Models\Teacher;
use Illuminate\Http\Request;
use App\Http\Requests\CardRequest;
use App\Models\Category;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

class CardController extends Controller
{
    public function index()
    {
        $pageInfo['title'] = 'البطاقات';
        $from = "2024-01-01T00:00:00";
        $to = now();
        if ((auth('admin')->check() && Auth::guard('admin')->user()->hasAllPermissions(['cards-list']))|| auth('partner')->check()){
            $cards = Card::with('category')->withSum('subscriptions', 'number_of_cards')->get();
            return view('admin.cards.index', compact('from','to','cards', 'pageInfo'));
        } elseif (auth('teacher')->check()) {
            $cards = Card::where('teacher_id' ,Auth::guard('teacher')->user()->id )->withSum('subscriptions', 'number_of_cards')->get();
            return view('admin.cards.index', compact('from','to','cards', 'pageInfo'));
        }else{
            return view('admin.pages.not_allowed', compact('pageInfo'));
        }
    }
    public function filter(Request $request)
    {

        $pageInfo['title'] = 'البطاقات';
        $from = $request->from;
        $to = $request->to;
        if ((auth('admin')->check() && Auth::guard('admin')->user()->hasAllPermissions(['cards-list']))|| auth('partner')->check()){
            $cards = Card::withSum('subscriptions', 'number_of_cards')->where('created_at', '>=' ,$from)->where('created_at', '<=' ,$to)->get();
            return view('admin.cards.index', compact('from','to','cards', 'pageInfo'));
        } elseif (auth('teacher')->check()) {
            $cards = Card::where('teacher_id' ,Auth::guard('teacher')->user()->id )->where('created_at', '>=' ,$from)->where('created_at', '<=' ,$to)->withSum('subscriptions', 'number_of_cards')->get();
            return view('admin.cards.index', compact('from','to','cards', 'pageInfo'));
        }else{
            return view('admin.pages.not_allowed', compact('pageInfo'));
        }
    }
        /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $pageInfo['title']= 'إضافة بطاقة';
        if (auth('admin')->check() && Auth::guard('admin')->user()->hasAllPermissions(['cards-add'])){
            $categories = Category::all();
            $teachers = Teacher::all();
            $courses = Course::where('teacher_id', 1 )->get();
            return view('admin.cards.create', compact('pageInfo','categories','teachers','courses'));
        }else{
            return view('admin.pages.not_allowed', compact('pageInfo'));
        }
    }


    public function store(CardRequest $request)
    {
        if (auth('admin')->check() && Auth::guard('admin')->user()->hasAllPermissions(['cards-add'])){
            try {
                //dd($request->all());
                DB::beginTransaction ();
                $card = Card::create([
                    'category_id'=>$request->category_id,
                    'teacher_id'=>$request->teacher_id,
                ]);
                if( $request->has('status') ){
                    $card->status="active";
                }else{
                    $card->status='inactive';
                }
                $card->save();
                if ($request->has('courses')) {
                    foreach ($request->courses as $courseId) {
                        $course = Course::find($courseId);
                        if ($course) {
                            $card->courses()->attach($course);
                        }
                    }
                }
                DB::commit();
                return to_route('cards.index')->with('message','تم اضافة البطاقة بنجاح');
            }catch(\Exception $e){
                Log::info($e->getMessage());
               // dd($e->getMessage());
                DB::rollBack();
                return to_route('cards.index')->with('error','حدث شيء ما خاطئ');
            }
        }else{
            return view('admin.pages.not_allowed', compact('pageInfo'));
        }
    }

        /**
     * Show the form for editing the specified resource.
     */
    public function edit($id)
    {
        $pageInfo['title']= 'تعديل البطاقة';
        if (auth('admin')->check() && Auth::guard('admin')->user()->hasAllPermissions(['cards-edit'])){
            $card = Card::find($id);
            $teachers = Teacher::all();
            $courses = Course::where('teacher_id', $card->teacher_id )->get();
            $categories = Category::all();
            return view('admin.cards.edit', compact('pageInfo', 'card','teachers','courses','categories'));
        }else{
            return view('admin.pages.not_allowed', compact('pageInfo'));
        }
    }

    public function update(CardRequest $request, $id)
    {
        $pageInfo['title']= 'تعديل البطاقة';
        if (auth('admin')->check() && Auth::guard('admin')->user()->hasAllPermissions(['cards-edit'])){
            try {
                DB::beginTransaction ();
                $card=Card::find($id);
                $card->update([
                    'category_id'=>$request->category_id,
                    'teacher_id'=>$card->teacher_id,
                ]);
                if( $request->has('status') ){
                    $card->status="active";
                }else{
                    $card->status='inactive';
                }

                $card->save();
                if ($request->has('courses')) {
                    foreach ($request->courses as $courseId) {
                        $course = Course::find($courseId);
                        if ($course) {
                            $card->courses()->sync($course);
                        }
                    }
                }
                DB::commit();
                return to_route('cards.index')->with('message','تم تعديل معلومات البطاقة');
            }catch(\Exception $e){
                Log::info($e->getMessage());
                dd($e->getMessage());
                DB::rollBack();
                return to_route('admin.cards.index')->with('error','حدث شيء ما خاطئ');
            }
        }else{
            return view('admin.pages.not_allowed', compact('pageInfo'));
        }

    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy( $id)
    {
        if (auth('admin')->check() && Auth::guard('admin')->user()->canAny(['cards-delete'])){
            try {
                $card=Card::find($id);
                //dd($card);
                if ($card->subscriptions->count() > 0){
                    return response()->json(['error' =>  'عذراً لا يمكن الحذف ، يوجد اشتراكات مرتبطة بهذه البطاقة']);
                }elseif($card->orders->count() > 0) {
                    return response()->json(['error' =>  'عذراً لا يمكن الحذف ، يوجد طلبات مرتبطة بهذه البطاقة']);
                }else {
                    $card->delete();
                    return response()->json([
                        'error'=>false,
                    ]);
                }
            }catch (\Exception $e){
                return response()->json([
                    'error'=>true,
                    'result'=>$e->getMessage(),
                ]);
            }
        }else{
            return response()->json(['error' =>  'لا تملك الإذن للحذف']);
        }


    }


    public function getCardSupplies($card)
    {

        try {
            $card = Card::where('id', $card)->first();
            if($card){
                $card = Card::where('id',$card->id)->with(['supplies'])->first();
               // dd($cards);
                return response()->json(['success' =>  $card]);
            } else {
                return response()->json(['error' =>  'البطاقة غير موجودة']);
            }
        }catch(\Exception $e){
            Log::info($e->getMessage());
          // dd($e->getMessage());
            DB::rollBack();
            return response()->json(['error' =>  'حدث شيء ما خاطئ']);
        }
    }
}

